/* ----------------------------------------------------------------------------
 * unijp_table.h
 * ----------------------------------------------------------------------------
 * Mastering programmed by YAMASHINA Hio
 *
 * Copyright 2008 YAMASHINA Hio
 * ----------------------------------------------------------------------------
 * This file is autogenerated by autogen.pl.
 * $Id$
 * ------------------------------------------------------------------------- */
#ifndef UNIJP_TABLE_H
#define UNIJP_TABLE_H

#ifdef __cplusplus
extern "C"
{
#endif

#include "unijp_types.h"

/* sizes. */
#define UJ_U2S_BYTES       85504
#define UJ_U2S_CHARS       42752
#define UJ_U2S_USIZE           2
#define UJ_S2U_BYTES       48573
#define UJ_S2U_CHARS       16191
#define UJ_S2U_USIZE           3
#define UJ_EI2U1_BYTES      2048
#define UJ_EI2U1_CHARS       512
#define UJ_EI2U1_USIZE         4
#define UJ_EU2I1_BYTES     16384
#define UJ_EU2I1_CHARS      8192
#define UJ_EU2I1_USIZE         2
#define UJ_EI2U2_BYTES      2048
#define UJ_EI2U2_CHARS       512
#define UJ_EI2U2_USIZE         4
#define UJ_EU2I2_BYTES     16384
#define UJ_EU2I2_CHARS      8192
#define UJ_EU2I2_USIZE         2
#define UJ_EJ2U1_BYTES      3072
#define UJ_EJ2U1_CHARS       768
#define UJ_EJ2U1_USIZE         4
#define UJ_EU2J1_BYTES     40960
#define UJ_EU2J1_CHARS      8192
#define UJ_EU2J1_USIZE         5
#define UJ_EJ2U2_BYTES      3072
#define UJ_EJ2U2_CHARS       768
#define UJ_EJ2U2_USIZE         4
#define UJ_EU2J2_BYTES     40960
#define UJ_EU2J2_CHARS      8192
#define UJ_EU2J2_USIZE         5
#define UJ_ED2U_BYTES       5120
#define UJ_ED2U_CHARS       1280
#define UJ_ED2U_USIZE          4
#define UJ_EU2D_BYTES      16384
#define UJ_EU2D_CHARS       8192
#define UJ_EU2D_USIZE          2
#define UJ_EA2U1_BYTES      1320
#define UJ_EA2U1_CHARS       660
#define UJ_EA2U1_USIZE         2
#define UJ_EU2A1_BYTES     16384
#define UJ_EU2A1_CHARS      8192
#define UJ_EU2A1_USIZE         2
#define UJ_EA2U2_BYTES      3288
#define UJ_EA2U2_CHARS      1644
#define UJ_EA2U2_USIZE         2
#define UJ_EU2A2_BYTES     16384
#define UJ_EU2A2_CHARS      8192
#define UJ_EU2A2_USIZE         2
#define UJ_EA2U1S_BYTES     4096
#define UJ_EA2U1S_CHARS     2048
#define UJ_EA2U1S_USIZE        2
#define UJ_EU2A1S_BYTES    16384
#define UJ_EU2A1S_CHARS     8192
#define UJ_EU2A1S_USIZE        2
#define UJ_EA2U2S_BYTES     4096
#define UJ_EA2U2S_CHARS     2048
#define UJ_EA2U2S_USIZE        2
#define UJ_EU2A2S_BYTES    16384
#define UJ_EU2A2S_CHARS     8192
#define UJ_EU2A2S_USIZE        2

/* data. */
extern const uj_uint8 _uj_table_u2s    [UJ_U2S_CHARS   ][UJ_U2S_USIZE   ];
extern const uj_uint8 _uj_table_s2u    [UJ_S2U_CHARS   ][UJ_S2U_USIZE   ];
extern const uj_uint8 _uj_table_ei2u1  [UJ_EI2U1_CHARS ][UJ_EI2U1_USIZE ];
extern const uj_uint8 _uj_table_eu2i1  [UJ_EU2I1_CHARS ][UJ_EU2I1_USIZE ];
extern const uj_uint8 _uj_table_ei2u2  [UJ_EI2U2_CHARS ][UJ_EI2U2_USIZE ];
extern const uj_uint8 _uj_table_eu2i2  [UJ_EU2I2_CHARS ][UJ_EU2I2_USIZE ];
extern const uj_uint8 _uj_table_ej2u1  [UJ_EJ2U1_CHARS ][UJ_EJ2U1_USIZE ];
extern const uj_uint8 _uj_table_eu2j1  [UJ_EU2J1_CHARS ][UJ_EU2J1_USIZE ];
extern const uj_uint8 _uj_table_ej2u2  [UJ_EJ2U2_CHARS ][UJ_EJ2U2_USIZE ];
extern const uj_uint8 _uj_table_eu2j2  [UJ_EU2J2_CHARS ][UJ_EU2J2_USIZE ];
extern const uj_uint8 _uj_table_ed2u   [UJ_ED2U_CHARS  ][UJ_ED2U_USIZE  ];
extern const uj_uint8 _uj_table_eu2d   [UJ_EU2D_CHARS  ][UJ_EU2D_USIZE  ];
extern const uj_uint8 _uj_table_ea2u1  [UJ_EA2U1_CHARS ][UJ_EA2U1_USIZE ];
extern const uj_uint8 _uj_table_eu2a1  [UJ_EU2A1_CHARS ][UJ_EU2A1_USIZE ];
extern const uj_uint8 _uj_table_ea2u2  [UJ_EA2U2_CHARS ][UJ_EA2U2_USIZE ];
extern const uj_uint8 _uj_table_eu2a2  [UJ_EU2A2_CHARS ][UJ_EU2A2_USIZE ];
extern const uj_uint8 _uj_table_ea2u1s [UJ_EA2U1S_CHARS][UJ_EA2U1S_USIZE];
extern const uj_uint8 _uj_table_eu2a1s [UJ_EU2A1S_CHARS][UJ_EU2A1S_USIZE];
extern const uj_uint8 _uj_table_ea2u2s [UJ_EA2U2S_CHARS][UJ_EA2U2S_USIZE];
extern const uj_uint8 _uj_table_eu2a2s [UJ_EU2A2S_CHARS][UJ_EU2A2S_USIZE];


#ifdef __cplusplus
}
#endif

#endif /* !defined(UNIJP_TABLE_H) */
/* ----------------------------------------------------------------------------
 * End of File.
 * ------------------------------------------------------------------------- */
